#version 130
#extension GL_EXT_gpu_shader3 : enable
#extension GL_EXT_gpu_shader4 : enable

uniform int lightNumber;
uniform int lightRecieve[7];

uniform sampler2D textureColorCurve;
uniform sampler2D textureImage[3];

uniform int fogMode;
uniform int useTexAlpha;
uniform int dissolveMode;
uniform int bFlat;

uniform vec2 ensembleParamA;
uniform float ensembleParamB;
uniform vec2 ensembleParamC;

uniform float projInv11;
uniform float projInv15;

in vec3 position;
in vec3 normal;
in vec4 color;
in vec4 colorDiffuse;
in vec4 colorSpecular;
flat in int blend;
//flat in int bFlat;
flat in int textureImageIndex;

//##FUNC_SMOOTHCLIPPING##

//##FUNC_ENSEMBLE_DITHERING##

//##FUNC_FOG_EFFECT##

//##FUNC_SPOT_EFFECT##

//##FUNC_NORMAL##

void main (void)
{
    vec4 texColor = vec4 (1, 1, 1, 1);
    
    switch(textureImageIndex){
    case 0:
	if (textureSize(textureImage[0], 0).x > 1)
	    texColor = texture2D(textureImage[0], gl_TexCoord[0].st);
	else
	    texColor = color;
	break;
    case 1:
	if (textureSize(textureImage[1], 0).x > 1)
	    texColor = texture2D(textureImage[1], gl_TexCoord[0].st);
	else
	    texColor = color;
	break;
    case 2:
	if (textureSize(textureImage[2], 0).x > 1)
	    texColor = texture2D(textureImage[2], gl_TexCoord[0].st);
	else
	    texColor = color;
	break;
    case 3:
	texColor = color;
    }

    float alpha = color[3];//gl_FrontLightProduct[0].ambient[3];
    /*
    if (useTexAlpha == 1){
	alpha *= texColor[3];
	if (alpha == 0) discard;
    }
    */
    
    float shininess = 20;
    vec3 fnormal = normalize(GetNormal());

    //##APPLY_SMOOTHCLIPPING##

    //##APPLY_ENSEMBLE_DITHERING##
    
    vec4 diffuseTotal = vec4(0, 0, 0, 1);
    vec4 specularTotal = vec4(0, 0, 0, 1);

    vec3 viewVec = -normalize(position);
    vec4 ambientClr = vec4(0, 0, 0, 0);

    for (int i = 0; i < lightNumber; i++){
	if (lightRecieve[i] == 0) continue;

	ambientClr += gl_LightSource[i].ambient * color;

	vec3 lightVec = gl_LightSource[i].position.xyz - position * gl_LightSource[i].position[3];
	float dis = length(lightVec);
	lightVec = normalize(lightVec);

	float diffuse = dot(lightVec, fnormal);
	
	if (diffuse > 0.0){
	    float spotAngleCos = dot(normalize(gl_LightSource[i].spotDirection),
				     -lightVec);
	    float lightFactor = GetSpotEffect(i, spotAngleCos);
	    
	    lightFactor /= (gl_LightSource[i].constantAttenuation +
			    gl_LightSource[i].linearAttenuation * dis +
			    gl_LightSource[i].quadraticAttenuation * dis * dis);

	    vec3 reflectVec = reflect(-lightVec, fnormal);
	    float specular = 
		pow(max(dot(viewVec, reflectVec), 0.0),
		    /*gl_FrontMaterial.shininess*/shininess) * lightFactor;
	    diffuseTotal += diffuse * vec4(gl_LightSource[i].diffuse.rgb, 0) * lightFactor;
	    specularTotal += specular * vec4(gl_LightSource[i].diffuse.rgb, 0);
	}
    }

    gl_FragColor = ambientClr +  
	colorDiffuse * diffuseTotal +
	colorSpecular * specularTotal;
    gl_FragColor *= texColor;
    
    FogEffect();

    if (alpha > 1.0){ 
	texColor[0] *= alpha;
	texColor[1] *= alpha;
	texColor[2] *= alpha; 
	alpha = 1.0;
    }

    switch(blend){
    case 0:
	//gl_FragColor.rgb *= alpha;
	gl_FragColor.a = alpha;
	break;
    case 1:
	//gl_FragColor.rgb *= alpha;
	gl_FragColor.a = 0;
	break;
    case 2:
	//gl_FragColor.rgb *= alpha;
	gl_FragColor.a = alpha * (1 - alpha);
	break;
    }
}
