//#extension GL_EXT_gpu_shader4 : enable

uniform sampler3D textureVolume;
uniform sampler1D textureCC;
uniform int lightNumber;
uniform float shadowAlpha;
uniform int shadowQuality;
uniform vec3 clipBoxMin;
uniform vec3 clipBoxMax;
uniform vec3 ambientClr;
uniform vec3 diffuseClr;
uniform vec3 specularClr;
uniform int shininess;
uniform float scatterFactor;
uniform int forceDiscard0;
uniform int trim;
uniform float intervalGlobal;
uniform int subQuality;

varying vec3 position;

//##FUNC_VOLUME_MASK_BEGIN##
bool GetMask(vec3 texCoord){
    return true;
}
//##FUNC_VOLUME_MASK_END##

//##FUNC_VOLUME_COORD_BEGIN##
vec3 GetDeformCoord(vec3 vecOrg){
    return vecOrg;
}

#define GUSEMATRIX 0
mat3 GetDeformNormalMatrix(vec3 texCoord, vec3 deformedCoord){
    return mat3(1, 0, 0, 0, 1, 0, 0, 0, 1);
}
//##FUNC_VOLUME_COORD_END##

//##FUNC_SPOT_EFFECT##

void main (void)
{
    gl_FragColor = vec4(0, 0, 0 ,0);

    float factorIntegrate = 1.0/subQuality;

    vec3 viewVec = -normalize(position);
    float angleMod = sqrt (1 + (viewVec[0]*viewVec[0]+viewVec[1]*viewVec[1])/viewVec[2]*viewVec[2]);
    vec3 viewVecGlobal = normalize(mat3(gl_ModelViewMatrixInverse) * viewVec) * intervalGlobal/subQuality * 3 * angleMod * angleMod;
    vec3 viewVecTexSpace = mat3(gl_TextureMatrix[0]) * viewVecGlobal;
    vec3 srcPos = vec3(gl_TextureMatrixInverse[0] * gl_TexCoord[0]);
    
    vec3 lightVecCache[8];
    float disCache[8];
    for (int i = 0; i < lightNumber; i++){
	lightVecCache[i] = gl_LightSource[i].position.xyz - position * gl_LightSource[i].position[3];
	disCache[i] = length(lightVecCache[i]);
	lightVecCache[i] = normalize(lightVecCache[i]);
    }

    int subCount = 0;
    for (int i = 0; i < subQuality; i++){

	vec3 tgtSubPos = srcPos + viewVecGlobal*i;

	if (any(lessThan(tgtSubPos, clipBoxMin))) continue;
	if (any(greaterThan(tgtSubPos, clipBoxMax))) continue;
	
	vec3 texCoord = vec3(gl_TexCoord[0]);
	vec3 deformedCoord = GetDeformCoord(gl_TexCoord[0]);
	if (GetMask(deformedCoord) == false) discard;

	if (trim == 1){
	    if (any(lessThan   (deformedCoord, vec3(0, 0, 0)))) continue;
	    if (any(greaterThan(deformedCoord, vec3(1, 1, 1)))) continue;
	}
    
	vec4 texColor = texture3D(textureVolume, deformedCoord);

	float value = texColor[0];
	if (forceDiscard0 == 1 && value == 0) discard;
    
	//vec4 texColor2 = texture1D(textureCC, value);
	vec4 color = texture1D(textureCC, value) * gl_Color;
	if (color.a == 0) continue;

	//ivec3 texSize = textureSize3D(textureVolume, 0);
	//float materialAlpha = gl_Color.a;

	//int kernelSize = 1;
	vec3 gradientL;
    
#if GUSEMATRIX
	vec3 gradient = vec3(textureOffset(textureVolume, deformedCoord, ivec3(-1, 0, 0)).r - textureOffset(textureVolume, deformedCoord, ivec3(1, 0, 0)).r,
			     textureOffset(textureVolume, deformedCoord, ivec3(0, -1, 0)).r - textureOffset(textureVolume, deformedCoord, ivec3(0, 1, 0)).r,
			     textureOffset(textureVolume, deformedCoord, ivec3(0, 0, -1)).r - textureOffset(textureVolume, deformedCoord, ivec3(0, 0, 1)).r);

	mat3 deformedMatrix = GetDeformNormalMatrix(texCoord, deformedCoord);
	vec3 deformedGradient = deformedMatrix * gradient;

	gradientL = vec3(gl_NormalMatrix * mat3(gl_TextureMatrixInverse[0]) * deformedGradient);
#else
        //ivec3 texSize = textureSize3D(textureVolume, 0);
	float delta = 0.01;
    
	vec3 deformedCoord_x1 = GetDeformCoord(texCoord + vec3(+delta, 0, 0));
	vec3 deformedCoord_x0 = GetDeformCoord(texCoord + vec3(-delta, 0, 0));
	vec3 deformedCoord_y1 = GetDeformCoord(texCoord + vec3(0, +delta, 0));
	vec3 deformedCoord_y0 = GetDeformCoord(texCoord + vec3(0, -delta, 0));
	vec3 deformedCoord_z1 = GetDeformCoord(texCoord + vec3(0, 0, +delta));
	vec3 deformedCoord_z0 = GetDeformCoord(texCoord + vec3(0, 0, -delta));
    
	gradientL = -vec3
	    (texture3D(textureVolume, deformedCoord_x1).r - 
	     texture3D(textureVolume, deformedCoord_x0).r,
	     texture3D(textureVolume, deformedCoord_y1).r - 
	     texture3D(textureVolume, deformedCoord_y0).r,
	     texture3D(textureVolume, deformedCoord_z1).r - 
	     texture3D(textureVolume, deformedCoord_z0).r); 
#endif
    
	vec3 fnormal = normalize(gradientL);
	//vec3 viewVec = -normalize(position);

	float scatterFactorAdjusted = 0.5 * (1+scatterFactor);
	for (int i = 0; i < lightNumber; i++){
	    float diffuse = 0;
	
	    vec3 lightVec = gl_LightSource[i].position.xyz - position * gl_LightSource[i].position[3];
	    float dis = length(lightVec);
	    lightVec = normalize(lightVec);

	    float spotAngleCos = dot(normalize(gl_LightSource[i].spotDirection),
				     -lightVec);
	    vec3 lightFactor = GetSpotEffect(i, spotAngleCos);
	    // W
	    lightFactor /= (gl_LightSource[i].constantAttenuation +
			    gl_LightSource[i].linearAttenuation * dis +
			    gl_LightSource[i].quadraticAttenuation * dis * dis);
	
	    vec3 lightVecGlobal = mat3(gl_ModelViewMatrixInverse) * lightVec / shadowQuality;
	    vec3 lightVecTexSpace = mat3(gl_TextureMatrix[0]) * lightVecGlobal;

	    //vec3 srcPos = vec3(gl_TextureMatrixInverse[0] * gl_TexCoord[0]);
	    //vec3 lightVecLocal = mat3(modelMatrix) * lightVecGlobal;

	    float maxDistance = 2;
	    if (gl_LightSource[i].position[3] == 1){
		float lightDistance = length(position - gl_LightSource[i].position.xyz);
		maxDistance = min(maxDistance, lightDistance);
	    }

	    int sampleNum = int(maxDistance * shadowQuality);
	    for (int i = 1; i <= sampleNum; i++){
	    
		vec3 tgtPos = tgtSubPos + lightVecGlobal * i;
		if (any(lessThan(tgtPos , clipBoxMin))) break;
		if (any(greaterThan(tgtPos, clipBoxMax))) break;
		vec3 tgtPosTexSpace = texCoord + lightVecTexSpace * i;
		vec3 tgtDeformCoord = GetDeformCoord(tgtPosTexSpace);
	
		float tgtValue = texture3D(textureVolume, tgtDeformCoord).r;
		float densityValue = texture1D(textureCC, tgtValue).a;
	    
		vec4 tgtClr = texture1D(textureCC, densityValue);
		if (tgtDeformCoord.z < 0 || tgtDeformCoord.z >= 1) tgtClr = vec4(0, 0, 0, 0);
		if (GetMask(tgtDeformCoord) == false) tgtClr = vec4(0, 0, 0, 0);
		if (forceDiscard0 == 1 && tgtValue == 0) tgtClr = vec4(0, 0, 0, 0);
	    
		lightFactor -= vec3(tgtClr) * tgtClr.a * shadowAlpha * scatterFactorAdjusted;
	    }
	
	    if (lightFactor.r < 0) lightFactor.r = 0;
	    if (lightFactor.g < 0) lightFactor.g = 0;
	    if (lightFactor.b < 0) lightFactor.b = 0;

	    diffuse = dot(lightVec, viewVec) * 0.5 * scatterFactor + 0.5;
	
	    if (diffuse < 0) diffuse = 0;
	    vec3 diffuseRGB = diffuse * lightFactor;

	    //˃xNg 
	    vec3 reflectVec = reflect(-lightVec, fnormal);
	    float specular = pow(max(dot(viewVec, reflectVec), 0.0), float(shininess)) * lightFactor;
	    if (specular < 0) specular = 0;
		
	    //vec4 color = texColor2 * gl_Color;
	
	    gl_FragColor.r +=
		color.r * ambientClr.r * gl_LightSource[i].ambient.r +
		color.r * diffuseRGB.r * diffuseClr.r * gl_LightSource[i].diffuse.r +
		specular * specularClr.r * gl_LightSource[i].specular.r;

	    gl_FragColor.g +=
		color.g * ambientClr.g * gl_LightSource[i].ambient.g +
		color.g * diffuseRGB.r * diffuseClr.g * gl_LightSource[i].diffuse.g +
		specular * specularClr.g * gl_LightSource[i].specular.g;

	    gl_FragColor.b +=
		color.b * ambientClr.b * gl_LightSource[i].ambient.b +
		color.b * diffuseRGB.r * diffuseClr.b * gl_LightSource[i].diffuse.b +
		specular * specularClr.b * gl_LightSource[i].specular.b;
	}

	//float alpha = texColor2.a * gl_Color.a;
	//if (alpha > 1) alpha = 1;

	//gl_FragColor[3] = alpha;

	gl_FragColor.a += color.a;
	subCount++;
    }

    if (gl_FragColor.a == 0) discard;

    float factorCount = 1.0f/subCount;

    gl_FragColor.r *= factorCount;
    gl_FragColor.g *= factorCount;
    gl_FragColor.b *= factorCount;
    gl_FragColor.a *= factorIntegrate;
    
    return;
}
